import _extends from "@babel/runtime/helpers/extends";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
const _excluded = ["row", "colNumber", "headings", "className"];
// eslint-disable-next-line import/no-extraneous-dependencies
import React from 'react';
import classNames from 'classnames';
import { Grid } from "../grid";
import { TableColumn } from "./tableColumn";
import { CodeHighlighter } from "../../baseUI/highlighter";
import { MarkdownLive } from "../../markdown/live";
import styles from "./tableRow.module.scss";
/**
 *
 * Renders a row in the table according to the order of the headings.
 */
export function TableRow(_ref) {
  let {
      row,
      colNumber = 4,
      headings,
      className
    } = _ref,
    rest = _objectWithoutProperties(_ref, _excluded);
  return /*#__PURE__*/React.createElement(Grid, _extends({
    col: colNumber,
    className: classNames(styles.propRow, className)
  }, rest), headings.map((title, index) => {
    if (title === 'name') {
      return /*#__PURE__*/React.createElement(TableColumn, {
        className: styles.breakWord,
        key: index
      }, /*#__PURE__*/React.createElement("div", {
        className: styles.columnContent
      }, /*#__PURE__*/React.createElement("div", {
        className: styles.name
      }, row[title]), row.required && /*#__PURE__*/React.createElement("div", {
        className: styles.required
      }, "(Required)"), row.version && /*#__PURE__*/React.createElement("div", {
        className: styles.version
      }, row.version)));
    }
    if (title === 'type' || title === 'default') {
      const text = row[title] || '-';
      const matches = text.match(/^\[([^\]]+)]\(([^)]+)\)$/);
      let columnContent = null;
      if (matches) {
        const [, typeText, href] = matches;
        columnContent = /*#__PURE__*/React.createElement("a", {
          className: styles.link,
          href: href
        }, typeText);
      } else {
        columnContent = /*#__PURE__*/React.createElement(CodeHighlighter, {
          language: "typescript",
          className: styles.highlighted
        }, text);
      }
      return /*#__PURE__*/React.createElement(TableColumn, {
        className: classNames(styles.breakWord, styles.typeColumn),
        key: index
      }, columnContent);
    }

    // default
    return /*#__PURE__*/React.createElement(TableColumn, {
      className: styles.breakWord,
      key: index
    }, /*#__PURE__*/React.createElement(MarkdownLive, {
      children: row[title]
    }));
  }));
}