// eslint-disable-next-line import/no-extraneous-dependencies
import React, { Fragment, useEffect, useMemo } from 'react';
// import debounce from lodash-es/debounce to enable tree-shaking
import debounce from 'lodash-es/debounce';
import { Pubsub } from '@arco-cli/aspect/dist/pubsub/previewRuntime';
import { MDXLayout } from "../markdown/mdxLayout";
import { Theme } from "./theme";
import { Content as DocContent } from "./doc/content";
import { PropertiesTable } from "./doc/propertiesTable";
import { DocAnchor } from "../markdown/components/docAnchor";
import { PreviewContextProvider } from "./previewContext";
import { VALID_MESSAGE_TYPE_FROM_PARENT_WINDOW } from "../constants";
import { SlotRegister } from "./SlotRegister";
import "../style/colors.scss";
import styles from "./app.module.scss";
export function App({
  doc,
  metadata,
  docContextProvider
}) {
  const {
    doclets,
    apiPlaceholderElementId
  } = metadata || {};
  const isEmpty = !doc && (!doclets || Array.isArray(doclets) && doclets.length === 0);
  const ContextProvider = typeof docContextProvider === 'function' ? docContextProvider : Fragment;
  const userEventListenerSlots = useMemo(() => {
    const slots = {};
    Object.values(VALID_MESSAGE_TYPE_FROM_PARENT_WINDOW).forEach(type => {
      slots[type] = new SlotRegister();
    });
    window.__registerArcoPreviewEventListener = (event, handler) => {
      var _slots$event;
      return (_slots$event = slots[event]) === null || _slots$event === void 0 ? void 0 : _slots$event.register(handler);
    };
    return slots;
  }, []);
  const {
    pubsub,
    pubsubTopic,
    pubsubTopicParent
  } = useMemo(() => {
    const pubsub = new Pubsub();
    const pubsubTopic = 'preview';
    const pubsubTopicParent = 'preview-host';
    try {
      // in iframe
      if (typeof window !== 'undefined' && window.self !== window.top) {
        new ResizeObserver(debounce(() => {
          pubsub.reportSize(pubsubTopic, {
            width: window.document.body.offsetWidth,
            height: window.document.body.offsetHeight
          });
        }, 300)).observe(document.body);
        window.addEventListener('hashchange', () => {
          pubsub.reportLocationHash(pubsubTopic, {
            hash: window.location.hash
          });
        });
      }
    } catch (err) {}
    return {
      pubsub,
      pubsubTopic,
      pubsubTopicParent
    };
  }, []);
  useEffect(() => {
    pubsub === null || pubsub === void 0 ? void 0 : pubsub.sub(pubsubTopicParent, message => {
      const {
        type,
        data
      } = message;
      switch (type) {
        case VALID_MESSAGE_TYPE_FROM_PARENT_WINDOW.switchDarkMode:
          {
            const body = document.body;
            data.dark ? body.setAttribute('arco-theme', 'dark') : body.removeAttribute('arco-theme');
            break;
          }
        case VALID_MESSAGE_TYPE_FROM_PARENT_WINDOW.appendExtraStyle:
          {
            const eleClassName = '__arco-component-extra-style';
            // clear all append styles at first
            document.querySelectorAll(`.${eleClassName}`).forEach(node => document.body.removeChild(node));
            // insert new stylesheet
            if (data.href) {
              var _document$body;
              const styleEle = document.createElement('link');
              styleEle.setAttribute('class', eleClassName);
              styleEle.setAttribute('type', 'text/css');
              styleEle.setAttribute('rel', 'stylesheet');
              styleEle.setAttribute('href', data.href);
              (_document$body = document.body) === null || _document$body === void 0 ? void 0 : _document$body.prepend(styleEle);
            }
            break;
          }
        case VALID_MESSAGE_TYPE_FROM_PARENT_WINDOW.scrollIntoView:
          {
            const {
              selector,
              options
            } = data;
            if (selector) {
              var _document$body$queryS;
              (_document$body$queryS = document.body.querySelector(selector)) === null || _document$body$queryS === void 0 ? void 0 : _document$body$queryS.scrollIntoView(options);
            }
            break;
          }
        default:
          break;
      }
      if (Object.values(VALID_MESSAGE_TYPE_FROM_PARENT_WINDOW).indexOf(type) > -1) {
        var _userEventListenerSlo;
        (_userEventListenerSlo = userEventListenerSlots[type]) === null || _userEventListenerSlo === void 0 ? void 0 : _userEventListenerSlo.values().forEach(callback => {
          callback(message);
        });
      }
    });
  }, [pubsub]);
  return /*#__PURE__*/React.createElement(PreviewContextProvider, {
    pubsub: pubsub,
    pubsubTopic: pubsubTopic,
    pubsubTopicParent: pubsubTopicParent
  }, /*#__PURE__*/React.createElement(Theme, null, /*#__PURE__*/React.createElement(MDXLayout, null, isEmpty ? /*#__PURE__*/React.createElement("div", {
    className: styles.emptyWarning
  }, /*#__PURE__*/React.createElement("div", {
    className: styles.title
  }, "No valid component overview found."), /*#__PURE__*/React.createElement("div", {
    className: styles.details
  }, "Please check your component [entries.preview] config in [arco.workspace.jsonc].")) : /*#__PURE__*/React.createElement(ContextProvider, null, /*#__PURE__*/React.createElement(DocContent, {
    doc: doc
  }), /*#__PURE__*/React.createElement(PropertiesTable, {
    doclet: doclets,
    placeholderID: apiPlaceholderElementId
  }), /*#__PURE__*/React.createElement(DocAnchor, null)))));
}